# Generated by Django 4.0.5 on 2022-09-28 06:33

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('triggers', '0006_action_model'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datachangetrigger',
            name='action',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='triggers.action'),
        ),
        migrations.AlterField(
            model_name='datefieldtrigger',
            name='action',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='triggers.action'),
        ),
        migrations.AlterField(
            model_name='datefieldtrigger',
            name='offset_amount',
            field=models.IntegerField(default=0, help_text='Can be negative (before) or positive (after)', verbose_name='Offset amount'),
        ),
        migrations.AlterField(
            model_name='datefieldtrigger',
            name='offset_type',
            field=models.CharField(choices=[('minutes', 'Minutes'), ('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('years', 'Years')], default='days', max_length=255, verbose_name='Offset type'),
        ),
    ]
