# Generated by Django 4.0.5 on 2022-09-28 05:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('triggers', '0005_alter_datachangetrigger_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='action',
            name='model',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='contenttypes.contenttype', verbose_name='Model'),
        ),
    ]
