# Generated by Django 4.0.5 on 2022-09-27 08:36

from django.db import migrations, models

import basxbread.contrib.publicurls.models


class Migration(migrations.Migration):

    dependencies = [
        ('publicurls', '0003_alter_publicurl_url'),
    ]

    operations = [
        migrations.AddField(
            model_name='publicurl',
            name='create_new_entry_from_response',
            field=models.CharField(blank=True, help_text="If 'Has Form' is set and the response is a redirect, the redirect URL is checked against this field (as regex). If it matches, a new public URL entry will automatically be created which allows the user to access the newly created and redirecte-to resource.", max_length=2048, validators=[basxbread.contrib.publicurls.models.validate_regex], verbose_name='Create new entry from response'),
        ),
    ]
