# Generated by Django 4.1.5 on 2023-02-04 08:11

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("customforms", "0002_alter_customformfield_customform_pdfimport_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pdfformfield",
            name="customform_field",
            field=models.ForeignKey(
                limit_choices_to={"customform": models.F("customform")},
                on_delete=django.db.models.deletion.CASCADE,
                to="customforms.customformfield",
            ),
        ),
    ]
