# Generated by Django 4.0.5 on 2022-10-01 08:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('triggers', '0007_alter_datachangetrigger_action_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datefieldtrigger',
            name='offset_type',
            field=models.CharField(choices=[('seconds', 'Seconds'), ('minutes', 'Minutes'), ('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('years', 'Years')], default='days', max_length=255, verbose_name='Offset type'),
        ),
    ]
