# Generated by Django 4.1.2 on 2023-01-13 04:03

from django.db import migrations, models

import basxbread.contrib.triggers.models


class Migration(migrations.Migration):

    dependencies = [
        ("triggers", "0009_alter_sendemail_email"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datachangetrigger",
            name="field",
            field=models.CharField(
                blank=True,
                help_text="Only trigger when a certain field has changed. Use comma to add multiple fields.",
                max_length=255,
                validators=[
                    basxbread.contrib.triggers.models.validate_direct_attributes
                ],
                verbose_name="Field",
            ),
        ),
        migrations.AlterField(
            model_name="datefieldtrigger",
            name="field",
            field=models.CharField(
                help_text="The field of the selected model which should trigger an action",
                max_length=255,
                validators=[
                    basxbread.contrib.triggers.models.validate_direct_attributes
                ],
                verbose_name="Field",
            ),
        ),
    ]
